/*                                                                               *\
    This file is part of the CBOLD(tm) framework for electronic design capture.
                         Copyright (C) 2007 Coolquest, Inc.

      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.

      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.

      You should have received a copy of the GNU General Public License along
      with this program; if not, write to the Free Software Foundation, Inc.,
      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    For more about the CBOLD framework and Coolquest, Inc., see www.cbold.com.
\*                                                                               */

// file cb_dummy_device.h:  class that generates dummy device files needed by some layout systems

#ifndef _cb_dummy_deviceH_
#define _cb_dummy_deviceH_

#include "cb_base.h"

// helper function for MakeDevice()
void MakeDevice_OutputPins( TPart* Part, ostream& Dest ) {
  size_t PinsOnLine = 0;
  size_t TotalPins  = 0;
  size_t PinCount   = Part->GetPackage().GetPinCount();

  for ( TCBBase* Member = Part->GetFirstMember(); Member != 0; Member = Member->Next ) {
    TPort* Port = dynamic_cast<TPort*>( Member );
    if ( Port == 0 )  continue;
    for ( int w = Port->GetMaxRange(); w >= Port->GetMinRange(); -- w ) {
      if ( PinsOnLine > 9 ) {
        Dest << ",\n";
        PinsOnLine = 0;
      }
      Dest << ' ';
      TPinSpec PinSpec( Port, w );
      PinSpec.OutputPins( Dest );
      if ( Port->MultiPinNode() ) {
        PinsOnLine += Port->GetPinCount();
        TotalPins  += Port->GetPinCount();
      }
      else {
        PinsOnLine ++;
        TotalPins  ++;
      }
    }
  }
  if ( TotalPins != PinCount )
    BEGERR << "MakeDeviceFiles(): TotalPins of " << TotalPins << " != PinCount of " << PinCount << " for part " << Part << ENDERR;
}

// make dummy device files for layout application
size_t MakeDeviceFiles( TModule* aRoot, char* DestDir ) {
  vector<TPart*> Parts;          // generate only one device file per part type

//*** form list of one example part instance per part type ***
  for ( TCBBase* CBB = aRoot->IterFirst(); CBB != 0; CBB = aRoot->IterNext() ) {
    TPart* Part = dynamic_cast<TPart*>( CBB );
    if ( Part ) {
      bool InList = false;
      for ( unsigned i = 0; i < Parts.size(); ++ i ) {
        if ( typeid( *Part ) == typeid( *Parts[ i ] ) ) {
          InList = true;
          break;
        }
      }
      if ( !InList )  Parts.push_back( Part );
    }
  }


//*** create one device file per listed part ***
  for ( unsigned i = 0; i < Parts.size(); ++ i ) {
    TPart*       Part     = Parts[ i ];
    const char*  TypeName = Part->GetTypeName();
    TPackage     Package  = Part->GetPackage();
    int          PinCount = Package.GetPinCount();
    string       DestPath = string( DestDir ) + string( "\\" ) + string( TypeName ) + string( ".txt" );

    ofstream Dest;
    Dest.open( DestPath.c_str(), ios_base::out );

    Dest << "(DEVICE FILE: " << Package.GetName() << ")";

    Dest << "\n\nPACKAGE " << Package.GetName();

    Dest << "\nCLASS ";
      if ( PinCount < 4 )  Dest << "DISCRETE";
      else                 Dest << "IC";

    Dest << "\nPINCOUNT " << PinCount;

    Dest << "\n\nPINORDER " << Package.GetName() << " ";
    MakeDevice_OutputPins( Part, Dest );

    Dest << "\n\nFUNCTION " << Package.GetName() << " " << Package.GetName() << " ";
    MakeDevice_OutputPins( Part, Dest );
//    Dest << "\n\nEND\n";

    Dest.close();
  }

  return Parts.size();
}

#endif

