/*                                                                               *\
    This file is part of the CBOLD(tm) framework for electronic design capture.
                         Copyright (C) 2007 Coolquest, Inc.

      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.

      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.

      You should have received a copy of the GNU General Public License along
      with this program; if not, write to the Free Software Foundation, Inc.,
      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    For more about the CBOLD framework and Coolquest, Inc., see www.cbold.com.
\*                                                                               */

// file cb_netlist_al3p.cpp:  netlist generator for Allegro layout tool (Allegro is a registered trademark of Cadence Design Systems, Inc.)

#include "cb_base.h"
#include "cb_shorthand.h"


// output standard netlist format (Allegro "third party" format)
void OutputCad3P( TRootModule* Root, ofstream& Dest, bool NullTimeStamp ) {
  TNetList* NL = &Root->NetList;
  Dest <<   "(NETLIST)";
  Dest << "\n(CREATED BY CBOLD)";
  if ( NullTimeStamp )  Dest << "\n(" << "Sat Jan 01 00:00:00 2000" << ')';
  else                  Dest << "\n(" <<  DateTimeString()          << ')';


// *** output part list ***//
  Dest << "\n$PACKAGES";

// get list of all parts in the design
  vector<TPart*> Parts;
  for ( TCBBase* CBB = Root->IterFirst(); CBB != 0; CBB = Root->IterNext() ) {
    TPart* Part = dynamic_cast<TPart*>( CBB );
    if ( Part ) Parts.push_back( Part );
  }
// format is:
// package ! part_name ; inst_ref ...
// SOIC14  ! T74als00  ; U1       U2

const int MaxRefsOnLine = 10;
  while ( ! Parts.empty() ) {      // empty list in process of outputing parts list
    vector<TPart*>::iterator I = Parts.begin();
    const char* PartName = (*I)->GetTypeName();    // use string comparison rather than typeid to check for other parts of same kind
    Dest << "\n\'" << (*I)->GetPackage() << "\' ! " << PartName << " ;";
    int RefsOnLine = 0;
    while ( I != Parts.end() ) {
      if ( strcmp( PartName, (*I)->GetTypeName() ) == 0 ) {
        if ( RefsOnLine >= MaxRefsOnLine ) {
          RefsOnLine = 0;          // write long list of refs to several lines
          Dest << " ,\n       ";   // add comma, newline, and indent
        }
        Dest << ' ' << (*I)->GetReferenceDesignator();
        I = Parts.erase( I );
        RefsOnLine ++;
      }
      else ++ I;
    }
  }


// *** output net list ***//
  Dest << "\n$NETS";

  for ( unsigned n = 0; n < NL->GetNetCount(); ++ n ) {   // for each net
    TNet* N         = NL->GetNet( n );
    if ( N->DesignatorEquals( NoConnect ) )  continue;    // discard no-connect net

    bool  GlobalNet = ( N->GetString()[ 0 ] == GlobalChar );
    Dest << '\n';
    if ( GlobalNet )  Dest << '\'';   // surround global net name in single quotes
    Dest << *N;                       // output net name
    if ( GlobalNet )  Dest << '\'';
    Dest << " ;";

    const int MaxPinsOnLine = 6;  // max pins to output on a single line
    int PinsOnLine = 0;
    for ( unsigned p = 0; p < N->GetPinSpecCount(); ++ p ) {
      TPinSpec PS = *N->GetPinSpec( p );

      TPart* OP = dynamic_cast<TPart*>( PS.GetPort()->GetOwner() );
      if ( OP == 0 )  continue;  // as of 14-Aug-2005, TNet's contain all ports, not just ports of parts
//    if ( OP == 0 )
//      BEGERR << "OutputCad3P(): port " << PS.GetPort() << " is owned by non-Part" << ENDERR;
      TDesignator Reference = OP->GetReferenceDesignator();

      size_t PinCount = PS.GetPinCount();
      if ( PinCount == 0 )
        BEGERR << "TModule::OutputCad3P():  PinSpec " << PS << " has no physical pin" << ENDERR;

      for ( unsigned i = 0; i < PinCount; ++ i ) {
        if ( PinsOnLine >= MaxPinsOnLine ) {
          PinsOnLine = 0;
          Dest << " ,\n       ";   // add comma, newline, and indent
        }
        Dest << ' ' << Reference << '.' << PS.GetPin( i );   // i is ignored except for multipin node
        PinsOnLine++;
      }
    }
  }

  Dest << "\n$END";
}

