/*                                                                               *\
    This file is part of the CBOLD(tm) framework for electronic design capture.
                         Copyright (C) 2007 Coolquest, Inc.

      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.

      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.

      You should have received a copy of the GNU General Public License along
      with this program; if not, write to the Free Software Foundation, Inc.,
      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    For more about the CBOLD framework and Coolquest, Inc., see www.cbold.com.
\*                                                                               */

// file cb_shorthand.h:  shorthand operators for making connections

#ifndef _cb_shorthandH_
#define _cb_shorthandH_


// The operators below may only be used within a TModule's Connect().
// The operators call ConnectingModule->conn().
// ConnectingModule is managed by TModule and is a pointer to the TModule whose
// Connect() is currently executing. See TModule::ConnectAll().

// The operators below return a char* when possible, because TModule::conn( char*...)
// will add connections to an existing SubBus, while TModule::conn( TPortRange...)
// will create a new SubBus.

// operator<<( const char* Prefix, TBundle& Bundle ) does not return a char* in order to prevent the user from doing something like this:
// "PREFIX_" << MyBundle << MyPort;
// If the operator had returned a char*, the above statement would be legal and equivalent to:
// "PREFIX_" << MyBundle;    // use "PREFIX_" as connection prefix -- ok
// "PREFIX_" << MyPort;      // use "PREFIX_" as connection name -- most likely not what user thought he was doing

#include "cb_base.h"
#include "cb_char_bucket.h"

TPortRange operator<<( TPortRange PortRangeA, TPortRange PortRangeB );
char* operator<<( char* CP, TPortRange PortRange  );
char* operator<<( TPortRange PortRange, char* CP );


// **** Operators for connecting ports of TBundle's **** //

// operator for connecting TBundle's ports to named nets
// the net names are derived from the user-specified prefix string as well as port names and sub-bundle names
// the actual character strings are maintained in a permanent string container
// it is legal for Prefix to begin with GlobalChar
TBundle& operator<<( const char* Prefix, TBundle& Bundle );

// operator for connecting TBundle's of identical type
TBundle& operator<<( TBundle& BundleA, TBundle& BundleB );


// **** Operators that accept a TModPar **** //
// When the TModPar is on the right of the ^, it must have one or two ports; its first port is connected.
// When the TModPar is on the left of the ^, it must have two ports; its second port is connected.
// operator^() is used rather than operator<<() because:
//  1. the separate syntax emphasizes that a part rather than a port is the operand
//  2. operator^() has lower precedence than operator<<(), e.g.,
//     "FOO" << Fred.Out   ^  Resistor1 ^   Bill.In << "BAR"  ;
//          is equivalenet to:
//   ( "FOO" << Fred.Out ) ^  Resistor1 ^ ( Bill.In << "BAR" );

TModPar& operator^( TPortRange PortRange, TModPar& ModPar );
TModPar& operator^( char* CP, TModPar& ModPar );
TModPar& operator^( TModPar& ModPar_L, TModPar& ModPar_R );
char* operator^( TModPar& ModPar, char* CP );
TPortRange operator^( TModPar& ModPar, TPortRange PortRange );

#endif

