/*                                                                               *\
    This file is part of the CBOLD(tm) framework for electronic design capture.
                         Copyright (C) 2007 Coolquest, Inc.

      This program is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation; either version 2 of the License, or
      (at your option) any later version.

      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.

      You should have received a copy of the GNU General Public License along
      with this program; if not, write to the Free Software Foundation, Inc.,
      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    For more about the CBOLD framework and Coolquest, Inc., see www.cbold.com.
\*                                                                               */

// file skeleton.h:  example skeleton design file

#ifndef _skeleton_h_                                  // <=== change these lines to match your file name
#define _skeleton_h_

#include "cb_base.h"
#include "cb_shorthand.h"

// above:  standard code
// below:  design-specific code

// This is the skeleton design main header file.                             <===
// To adapt this file to your design, see comments that contain this arrow:  <===
// See also skeleton.cpp and skel_sub.h.                                     <===

// file paths, etc.
#define  DESIGN_NAME  "skeleton"                      // <=== name of design -- used primarily to form file names
//                    dir for all CBOLD output  subdir. for this design   base of file name
#define  OUTPUT_BASE  OUTPUT_PATH DIR_SEP       DESIGN_NAME DIR_SEP       DESIGN_NAME        // concatenation of path and name --> base for output file names

// <=== Be sure that directory OUTPUT_BASE exists, otherwise a runtime error will result.
// <=== For the skeleton design, OUTPUT_BASE is cbold/output/skeleton.
// <=== To see/change OUTPUT_PATH, see cb_user_pref.h.

// misc. header files, if needed
//#include <locale>                                   // <=== example standard header file

// header files needed by many subsystems
#include "decoupling.h"                               // <=== example header file -- be sure the included file's directory is in your compiler's include path

// subsystem header files
#include "skel_sub.h"                                 // <=== example subsystem header file -- be sure the included file's directory is in your compiler's include path

// ************ subsystems ************               // <=== design summary comment
// subsystem           abbreviation
// ------------        ------------
// Power               PW                             // <=== example subsystem


class CM_Root : public TRootModule {
public:
// root-level module has no ports

// ***** member subsystems ***** //

  CM_Power            Power;                          // <=== example subsystem

  virtual void Register() {
    reg( Power );  Power.SetReferenceBase( "PW" );    // <=== example subsystem registration with optional setting of reference designator base
  }

  virtual void Connect() {
                                                      // <=== make root-level connections here, e.g., connections between interface bundles
    wireall( "VCC"  );                                // <=== example power connections
    wireall( "GND" );
  }
};

#endif

